/*

    dsh-observable-graph  Observable decorators for graph interfaces.
    Copyright (c) 2008-2012 held jointly by the individual authors.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 3 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; with out even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
    License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library;  if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.

    > http://www.fsf.org/licensing/licenses/lgpl.html
    > http://www.opensource.org/licenses/lgpl-license.php

*/
package org.dishevelled.observable.graph.event;

import java.util.EventListener;

/**
 * A listener that receives notification of changes made to an observable graph.
 *
 * @param <N> node value type
 * @param <E> edge value type
 * @author  Michael Heuer
 * @version $Revision$ $Date$
 */
public interface GraphChangeListener<N, E>
    extends EventListener
{

    /**
     * Notify this listener that the observable graph has been cleared.
     *
     * @param e graph change event
     */
    void cleared(GraphChangeEvent<N, E> e);

    /**
     * Notify this listener that a node has been created in the observable graph.
     *
     * @param e graph change event
     */
    void nodeCreated(GraphChangeEvent<N, E> e);

    /**
     * Notify this listener that an edge has been created in the observable graph.
     *
     * @param e graph change event
     */
    void edgeCreated(GraphChangeEvent<N, E> e);

    /**
     * Notify this listener than a node has been removed from the observable graph.
     *
     * @param e graph change event
     */
    void nodeRemoved(GraphChangeEvent<N, E> e);

    /**
     * Notify this listener that an edge has been removed from the observable graph.
     *
     * @param e graph change event
     */
    void edgeRemoved(GraphChangeEvent<N, E> e);
}